@@ -8,7 +8,7 @@
 #   The Artistic License 2.0 (GPL Compatible)
 #
 package Redis;
-$Redis::VERSION = '1.982';
+$Redis::VERSION = '1.991';
 # ABSTRACT: Perl binding for Redis database
 # VERSION
 # AUTHORITY
@@ -34,6 +34,7 @@ use constant WIN32       => $^O =~ /mswin32/i;
 use constant EWOULDBLOCK => eval {Errno::EWOULDBLOCK} || -1E9;
 use constant EAGAIN      => eval {Errno::EAGAIN} || -1E9;
 use constant EINTR       => eval {Errno::EINTR} || -1E9;
+use constant ECONNRESET  => eval {Errno::ECONNRESET} || -1E9;
 use constant BUFSIZE     => 4096;
 
 sub _maybe_enable_timeouts {
@@ -329,7 +330,7 @@ sub quit {
     $self->__send_command('QUIT');
   };
 
-  $self->__close_sock() if $self->{sock};
+  $self->__close_sock();
 
   return 1;
 }
@@ -620,8 +621,9 @@ sub __build_sock {
   if (exists $self->{password}) {
     try { $self->auth($self->{password}) }
     catch {
+      my $error = $_;
       $self->{reconnect} = 0;
-      croak("Redis server refused password");
+      croak('Redis server authentication error: ' . $error);
     };
   }
 
@@ -635,6 +637,7 @@ sub __close_sock {
   $self->{__buf} = '';
   delete $self->{__inside_watch};
   delete $self->{__inside_transaction};
+  defined $self->{sock} or return 1;
   return close(delete $self->{sock});
 }
 
@@ -777,8 +780,12 @@ sub __read_line {
   my $sock = $self->{sock};
 
   my $data = $self->__read_line_raw;
-  croak("Error while reading from Redis server: $!")
-    unless defined $data;
+  if (! defined $data) {
+      # In case the caller catches the exception and wants to persist on using
+      # the redis connection, let's forbid that.
+      $self->__close_sock();
+      croak("Error while reading from Redis server: $!")
+  }
 
   chomp $data;
   warn "[RECV RAW] '$data'" if $self->{debug};
@@ -868,6 +875,9 @@ sub __try_read_sock {
       ## Keep going if nothing there, but socket is alive
       return 0 if $err and ($err == EWOULDBLOCK or $err == EAGAIN);
 
+      ## on freebsd, if we got ECONNRESET, it's a timeout from the other side
+      return 0 if ($err && $err == ECONNRESET && $^O eq 'freebsd');
+
       ## result is undef but err is 0? should never happen
       return if $err == 0;
 
@@ -905,7 +915,7 @@ Redis - Perl binding for Redis database
 
 =head1 VERSION
 
-version 1.982
+version 1.991
 
 =head1 SYNOPSIS
 
@@ -933,7 +943,7 @@ version 1.982
     ## Die if you can't after that
     my $redis = Redis->new(reconnect => 60, every => 1_000_000);
 
-    ## Try each 100ms upto 2 seconds (every is in microseconds)
+    ## Try each 100ms up to 2 seconds (every is in microseconds)
     my $redis = Redis->new(reconnect => 2, every => 100_000);
 
     ## Enable connection timeout (in seconds)
